function deriv = iterateDEs(t, var)
    % Function to iterate the set of PDE's given in equations 7,8 and 9 in
    % the write up.
    %
    % t:    desired time (seconds)
    % var:  a set of the temperature profiles (snow then ice) at the discretized positions along with the
    %       ice depth. var=[snow_temp_profile ice_temp_profile ice_depth]
    
    %In this function, h_ice and h_snow is the reparamterized depth variable, living in
    %[0, 1]
    
    
    global num_of_points_ice num_of_points_snow T_freezing delta_h_ice delta_h_snow V_a;
    
    temp_snow = var(1:num_of_points_snow);
    temp_ice = var(num_of_points_snow+1:num_of_points_snow+num_of_points_ice);
    ice_depth = var(num_of_points_snow+num_of_points_ice+1);
    
    %fetch the current air temperature
    air_temp = calcAirTemp(t);
    
    %snow thickness from the input data (user specified)
    snow_thickness = snowThickness(t);
    snow_growth_rate = snowGrowthRate(t);
    
    %initialize response variables
    dTice_dt = zeros(length(temp_ice),1);
    dTsnow_dt = zeros(length(temp_snow), 1);
    
    %Calculate the advancement of the boundary as per the boundary growth
    %equation 9
    interface_salinity = calcSalinity(ice_depth);
    dT_dz = (T_freezing-temp_ice(end))/delta_h_ice * 1/ice_depth;
    ocean_flux = calcOceanFlux(t);
    growth_rate = kIce(T_freezing, interface_salinity)/(rhoIce(T_freezing, interface_salinity, V_a)*LIce(T_freezing, interface_salinity))*dT_dz + ocean_flux/(LIce(T_freezing, interface_salinity) * rhoIce(T_freezing, interface_salinity, V_a));
    
    %If no snow is present ignore the effects of snow. Note the use of a
    %small parameter here as numerically we have trouble with exactly zero
    %snow.
    if snow_thickness > 10^-9 
        T_snow_ice = surfaceTemp(temp_snow(1), temp_ice(1), snow_thickness, ice_depth, t);
    else
        T_snow_ice = air_temp;
    end
    
    %iterate the snow then ice temperature profile according to equations 7
    %and 8
    %Note a slight deviation from the write up. The thermal conductivity of
    %sea ice/snow depends on the temperature and the salinity (i.e. the depth,
    %the salinity is calculated via the user specified salinity profile). 
    %Here to estimate the thermal conductivity in between sample points we 
    %linearly interpolate on the temperature and salinity and evaluate the 
    %thermal conductiviy at that point. In the write up the thermal conductivity 
    %itself was interpolated.
    for i=1:length(temp_snow)
        h_snow = i/(length(temp_snow)+1);
        cp_z = cSnow(temp_snow(i)) * rhoSnow();
        
        if i==1
            k_snow_plus = kSnow(0.5*(temp_snow(i)+temp_snow(i+1)));
            k_snow_minus = kSnow(0.5*(temp_snow(i)+T_snow_ice));
            dTsnow_dt(i) = 1/(cp_z*snow_thickness^2*delta_h_snow^2)*(k_snow_plus*temp_snow(i+1)-(k_snow_minus+k_snow_plus)*temp_snow(i)+k_snow_minus*T_snow_ice) + h_snow/snow_thickness *(temp_snow(i+1)-T_snow_ice)/(2*delta_h_snow)*snow_growth_rate;
        elseif i==length(temp_snow)
            k_snow_plus = kSnow(0.5*(temp_snow(i)+air_temp));
            k_snow_minus = kSnow(0.5*(temp_snow(i)+temp_snow(i-1)));
            dTsnow_dt(i) = 1/(cp_z*snow_thickness^2*delta_h_snow^2)*(k_snow_plus*air_temp-(k_snow_plus+k_snow_minus)*temp_snow(i)+k_snow_minus*temp_snow(i-1)) + h_snow/snow_thickness *(air_temp-temp_snow(i-1))/(2*delta_h_snow)*snow_growth_rate;
        else
            k_snow_plus = kSnow(0.5*(temp_snow(i)+temp_snow(i+1)));
            k_snow_minus = kSnow(0.5*(temp_snow(i)+temp_snow(i-1)));
            dTsnow_dt(i) = 1/(cp_z*snow_thickness^2*delta_h_snow^2)*(k_snow_plus*temp_snow(i+1)-(k_snow_plus+k_snow_minus)*temp_snow(i)+k_snow_minus*temp_snow(i-1)) + h_snow/snow_thickness *(temp_snow(i+1)-temp_snow(i-1))/(2*delta_h_snow)*snow_growth_rate;
        end
    end
    
    for i=1:length(temp_ice)
        h = i/(length(temp_ice)+1);
        cp_z = cIce(temp_ice(i), calcSalinity(h*ice_depth)) * rhoIce(temp_ice(i), calcSalinity(h*ice_depth), V_a);
        
        if i==1
            k_ice_plus = kIce(0.5*(temp_ice(i)+temp_ice(i+1)), calcSalinity(ice_depth*(h+0.5)));
            k_ice_minus = kIce(0.5*(temp_ice(i)+T_snow_ice), calcSalinity(ice_depth*(h-0.5)));
            dTice_dt(i) = h/ice_depth *(temp_ice(i+1)-T_snow_ice)/(2*delta_h_ice)*growth_rate + 1/(cp_z)*1/(ice_depth^2)*1/(delta_h_ice^2)*(k_ice_minus*T_snow_ice-(k_ice_plus+k_ice_minus)*temp_ice(i)+k_ice_plus*temp_ice(i+1));
        elseif i==length(temp_ice)
            k_ice_plus = kIce(0.5*(temp_ice(i)+T_freezing), calcSalinity(ice_depth*(h+0.5)));
            k_ice_minus = kIce(0.5*(temp_ice(i)+temp_ice(i-1)), calcSalinity(ice_depth*(h-0.5)));
            dTice_dt(i) = h/ice_depth *(T_freezing-temp_ice(i-1))/(2*delta_h_ice)*growth_rate + 1/(cp_z)*1/(ice_depth^2)*1/(delta_h_ice^2)*(k_ice_minus*temp_ice(i-1)-(k_ice_plus+k_ice_minus)*temp_ice(i)+k_ice_plus*T_freezing);
        else
            k_ice_plus = kIce(0.5*(temp_ice(i)+temp_ice(i+1)), calcSalinity(ice_depth*(h+0.5)));
            k_ice_minus = kIce(0.5*(temp_ice(i)+temp_ice(i-1)), calcSalinity(ice_depth*(h-0.5)));
            dTice_dt(i) = h/ice_depth *(temp_ice(i+1)-temp_ice(i-1))/(2*delta_h_ice)*growth_rate + 1/(cp_z)*1/(ice_depth^2)*1/(delta_h_ice^2)*(k_ice_minus*temp_ice(i-1)-(k_ice_plus+k_ice_minus)*temp_ice(i)+k_ice_plus*temp_ice(i+1));
        end
    end
    
    %the response variable is the rate of change of each variable (the
    %snow/ice temperature profiles and the ice depth)
    deriv = [dTsnow_dt; dTice_dt; growth_rate];
end
